@@ -92,7 +92,7 @@ static const char	xml_friendly_chars[257] = "\
 
 static const char	xml_quote_chars[257] = "\
 :::::::::11::1::::::::::::::::::\
-11611151111111111111111111111111\
+11611151111111111111111111114141\
 11111111111111111111111111111111\
 11111111111111111111111111111111\
 11111111111111111111111111111111\
@@ -194,7 +194,7 @@ fill_value(Out out, const char *value, size_t len) {
 	memcpy(out->cur, value, len);
 	out->cur += len;
     } else {
-	for (; '\0' != *value; value++) {
+	for (; 0 < len; len--, value++) {
 	    *out->cur++ = *value;
 	}
     }
@@ -210,7 +210,7 @@ fill_attr(Out out, char name, const char *value, size_t len) {
 	memcpy(out->cur, value, len);
 	out->cur += len;
     } else {
-	for (; '\0' != *value; value++) {
+	for (; 0 < len; len--, value++) {
 	    *out->cur++ = *value;
 	}
     }
@@ -331,7 +331,7 @@ dump_value(Out out, const char *value, size_t size) {
 	memcpy(out->cur, value, size);
 	out->cur += size;
     } else {
-	for (; '\0' != *value; value++) {
+	for (; 0 < size; size--, value++) {
 	    *out->cur++ = *value;
 	}
     }
@@ -345,7 +345,7 @@ dump_str_value(Out out, const char *value, size_t size, const char *table) {
     if (out->end - out->cur <= (long)xsize) {
 	grow(out, xsize);
     }
-    for (; '\0' != *value; value++) {
+    for (; 0 < size; size--, value++) {
 	if ('1' == table[(uchar)*value]) {
 	    *out->cur++ = *value;
 	} else {
@@ -1270,6 +1270,7 @@ dump_obj_to_xml(VALUE obj, Options copts, Out out) {
     out->circ_cnt = 0;
     out->opts = copts;
     out->obj = obj;
+    *out->cur = '\0';
     if (Yes == copts->circular) {
 	ox_cache8_new(&out->circ_cache);
     }
@@ -1284,7 +1285,9 @@ dump_obj_to_xml(VALUE obj, Options copts, Out out) {
 	out->w_end = dump_end;
 	dump_first_obj(obj, out);
     }
-    dump_value(out, "\n", 1);
+    if (0 <= out->indent) {
+	dump_value(out, "\n", 1);
+    }
     if (Yes == copts->circular) {
 	ox_cache8_delete(out->circ_cache);
     }
