@@ -1,31 +1,6 @@
 /* sax_buf.c
  * Copyright (c) 2011, Peter Ohler
  * All rights reserved.
- * 
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions are met:
- * 
- *  - Redistributions of source code must retain the above copyright notice, this
- *    list of conditions and the following disclaimer.
- * 
- *  - Redistributions in binary form must reproduce the above copyright notice,
- *    this list of conditions and the following disclaimer in the documentation
- *    and/or other materials provided with the distribution.
- * 
- *  - Neither the name of Peter Ohler nor the names of its contributors may be
- *    used to endorse or promote products derived from this software without
- *    specific prior written permission.
- * 
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
- * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
- * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
- * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
- * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
- * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
- * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
- * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
 #include <stdlib.h>
@@ -49,49 +24,32 @@ static VALUE		rescue_cb(VALUE rdr, VALUE err);
 static VALUE		io_cb(VALUE rdr);
 static VALUE		partial_io_cb(VALUE rdr);
 static int		read_from_io(Buf buf);
-#ifndef JRUBY_RUBY
 static int		read_from_fd(Buf buf);
-#endif
 static int		read_from_io_partial(Buf buf);
 static int		read_from_str(Buf buf);
 
 void
 ox_sax_buf_init(Buf buf, VALUE io) {
-    if (ox_stringio_class == rb_obj_class(io)) {
-	VALUE	s = rb_funcall2(io, ox_string_id, 0, 0);
+    volatile VALUE	io_class = rb_obj_class(io);
+    VALUE		rfd;
+
+    if (rb_cString == io_class) {
+	buf->read_func = read_from_str;
+	buf->in.str = StringValuePtr(io);
+    } else if (ox_stringio_class == io_class && 0 == FIX2INT(rb_funcall2(io, ox_pos_id, 0, 0))) {
+	volatile VALUE	s = rb_funcall2(io, ox_string_id, 0, 0);
 
 	buf->read_func = read_from_str;
-	buf->in_str = StringValuePtr(s);
+	buf->in.str = StringValuePtr(s);
+    } else if (rb_cFile == io_class && Qnil != (rfd = rb_funcall(io, ox_fileno_id, 0))) {
+	buf->read_func = read_from_fd;
+	buf->in.fd = FIX2INT(rfd);
     } else if (rb_respond_to(io, ox_readpartial_id)) {
-#ifdef JRUBY_RUBY
 	buf->read_func = read_from_io_partial;
-	buf->io = io;
-#else
-        VALUE   rfd;
-
-        if (rb_respond_to(io, ox_fileno_id) && Qnil != (rfd = rb_funcall(io, ox_fileno_id, 0))) {
-            buf->read_func = read_from_fd;
-            buf->fd = FIX2INT(rfd);
-        } else {
-            buf->read_func = read_from_io_partial;
-            buf->io = io;
-        }
-#endif
+	buf->in.io = io;
     } else if (rb_respond_to(io, ox_read_id)) {
-#ifdef JRUBY_RUBY
 	buf->read_func = read_from_io;
-	buf->io = io;
-#else
-        VALUE   rfd;
-
-        if (rb_respond_to(io, ox_fileno_id) && Qnil != (rfd = rb_funcall(io, ox_fileno_id, 0))) {
-            buf->read_func = read_from_fd;
-            buf->fd = FIX2INT(rfd);
-        } else {
-            buf->read_func = read_from_io;
-            buf->io = io;
-        }
-#endif
+	buf->in.io = io;
     } else {
         rb_raise(ox_arg_error_class, "sax_parser io argument must respond to readpartial() or read().\n");
     }
@@ -102,8 +60,10 @@ ox_sax_buf_init(Buf buf, VALUE io) {
     buf->read_end = buf->head;
     buf->pro = 0;
     buf->str = 0;
+    buf->pos = 0;
     buf->line = 1;
     buf->col = 0;
+    buf->pro_pos = 1;
     buf->pro_line = 1;
     buf->pro_col = 0;
     buf->dr = 0;
@@ -113,7 +73,7 @@ int
 ox_sax_buf_read(Buf buf) {
     int         err;
     size_t      shift = 0;
-    
+
     // if there is not much room to read into, shift or realloc a larger buffer.
     if (buf->head < buf->tail && 4096 > buf->end - buf->tail) {
         if (0 == buf->pro) {
@@ -160,21 +120,14 @@ ox_sax_buf_read(Buf buf) {
 
 static VALUE
 rescue_cb(VALUE rbuf, VALUE err) {
-#ifndef JRUBY_RUBY
-    /* JRuby seems to play by a different set if rules. It passes in an Fixnum
-     * instead of an error like other Rubies. For now assume all errors are
-     * EOF and deal with the results further down the line. */
-#if (defined(RUBINIUS_RUBY) || (1 == RUBY_VERSION_MAJOR && 8 == RUBY_VERSION_MINOR))
-    if (rb_obj_class(err) != rb_eTypeError) {
-#else
-    if (rb_obj_class(err) != rb_eEOFError) {
-#endif
+    VALUE	err_class = rb_obj_class(err);
+
+    if (err_class != rb_eTypeError && err_class != rb_eEOFError) {
 	Buf	buf = (Buf)rbuf;
 
         //ox_sax_drive_cleanup(buf->dr); called after exiting protect
         rb_raise(err, "at line %d, column %d\n", buf->line, buf->col);
     }
-#endif
     return Qfalse;
 }
 
@@ -187,10 +140,10 @@ partial_io_cb(VALUE rbuf) {
     size_t	cnt;
 
     args[0] = ULONG2NUM(buf->end - buf->tail);
-    rstr = rb_funcall2(buf->io, ox_readpartial_id, 1, args);
+    rstr = rb_funcall2(buf->in.io, ox_readpartial_id, 1, args);
     str = StringValuePtr(rstr);
     cnt = strlen(str);
-    //printf("*** read %lu bytes, str: '%s'\n", cnt, str);
+    //printf("*** read partial %lu bytes, str: '%s'\n", cnt, str);
     strcpy(buf->tail, str);
     buf->read_end = buf->tail + cnt;
 
@@ -206,10 +159,10 @@ io_cb(VALUE rbuf) {
     size_t	cnt;
 
     args[0] = ULONG2NUM(buf->end - buf->tail);
-    rstr = rb_funcall2(buf->io, ox_read_id, 1, args);
+    rstr = rb_funcall2(buf->in.io, ox_read_id, 1, args);
     str = StringValuePtr(rstr);
     cnt = strlen(str);
-    /*printf("*** read %lu bytes, str: '%s'\n", cnt, str); */
+    //printf("*** read %lu bytes, str: '%s'\n", cnt, str);
     strcpy(buf->tail, str);
     buf->read_end = buf->tail + cnt;
 
@@ -226,13 +179,12 @@ read_from_io(Buf buf) {
     return (Qfalse == rb_rescue(io_cb, (VALUE)buf, rescue_cb, (VALUE)buf));
 }
 
-#ifndef JRUBY_RUBY
 static int
 read_from_fd(Buf buf) {
     ssize_t     cnt;
     size_t      max = buf->end - buf->tail;
 
-    cnt = read(buf->fd, buf->tail, max);
+    cnt = read(buf->in.fd, buf->tail, max);
     if (cnt < 0) {
         ox_sax_drive_error(buf->dr, "failed to read from file");
         return -1;
@@ -241,7 +193,6 @@ read_from_fd(Buf buf) {
     }
     return 0;
 }
-#endif
 
 static char*
 ox_stpncpy(char *dest, const char *src, size_t n) {
@@ -262,14 +213,14 @@ read_from_str(Buf buf) {
     char	*s;
     long	cnt;
 
-    if ('\0' == *buf->in_str) {
+    if ('\0' == *buf->in.str) {
 	/* done */
 	return -1;
     }
-    s = ox_stpncpy(buf->tail, buf->in_str, max);
+    s = ox_stpncpy(buf->tail, buf->in.str, max);
     *s = '\0';
     cnt = s - buf->tail;
-    buf->in_str += cnt;
+    buf->in.str += cnt;
     buf->read_end = buf->tail + cnt;
 
     return 0;
